﻿#region License
// Copyright 2009 Buu Nguyen (http://www.buunguyen.net/blog)
// 
// Licensed under the Apache License, Version 2.0 (the "License"); 
// you may not use this file except in compliance with the License. 
// You may obtain a copy of the License at 
// 
// http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, 
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
// See the License for the specific language governing permissions and 
// limitations under the License.
// 
// The latest version of this file can be found at http://dotcommon.codeplex.com
#endregion

using System;
using System.Globalization;

namespace DotCommon.Reflection
{
    public static class TypeExtensions
    {
        /// <summary>
        /// Converts <paramref name="value"/> to the corresponding value of 
        /// type <paramref name="targetType"/>.  <paramref name="targetType"/>
        /// must be one of the following types: enum, nullable, primitive 
        /// <see cref="Convert.ChangeType(object,System.Type)"/>.
        /// </summary>
        /// <param name="value">The string to be converted</param>
        /// <param name="targetType">The target type to convert <paramref name="value"/> to</param>
        /// <returns>The corresponding value, in <paramref name="targetType"/>, of <paramref name="value"/></returns>
        public static object ConvertToType(this string value, Type targetType)
        {
            if (targetType == typeof(string))
                return value;
            if (targetType.IsEnum)
                return Enum.Parse(targetType, value);
            if (targetType.IsNullable())
                targetType = Nullable.GetUnderlyingType(targetType);
            return Convert.ChangeType(value, targetType, CultureInfo.InvariantCulture);
        }

        /// <summary>
        /// Converts <paramref name="value"/> to the corresponding value of 
        /// type <paramref name="targetType"/>.  <paramref name="targetType"/>
        /// must be one of the following types: enum, nullable, primitive 
        /// <see cref="Convert.ChangeType(object,System.Type)"/>.
        /// </summary>
        /// <param name="value">The string to be converted</param>
        /// <param name="targetType">The target type to convert <paramref name="value"/> to</param>
        /// <param name="defaultValue">The default value to be used if conversion fails</param>
        /// <returns>The corresponding value, in <paramref name="targetType"/>, of <paramref name="value"/></returns>
        public static object ConvertToType(this string value, Type targetType, object defaultValue)
        {
            try
            {
                return ConvertToType(value, targetType);    
            } 
            catch
            {
                return defaultValue;
            }
        }

        /// <summary>
        /// Checks whether <paramref name="type"/> is a nullable type.
        /// </summary>
        /// <param name="type">A type</param>
        /// <returns>true if <paramref name="type"/> is nullable.  False otherwise.</returns>
        public static bool IsNullable(this Type type)
        {
            if (!type.IsValueType) 
                return false;
            if (Nullable.GetUnderlyingType(type) != null) 
                return true;
            return false;
        }
    }
}